@@ -1,7 +1,7 @@
-from src.core.network import Network
+from .src.core.network import Network
 
 
 class Livefyre(object):
     @staticmethod
     def get_network(network_name, network_key):
-        return Network(network_name, network_key)
\ No newline at end of file
+        return Network.init(network_name, network_key)
\ No newline at end of file
